#!/bin/env python
###	ssehunter.py	Matthew Baker 09/2003

#N ssehunter.py
#F secondary structure identification

import os
import sys
import string
import commands
import math
import EMAN
from math import *
from sys import argv
import Numeric
import MLab
#import Pycluster

def cross_product(a,b):
    """Cross product of two 3-d vectors. from http://www-hep.colorado.edu/~fperez/python/python-c/weave_examples.html
    """
    cross = [0]*3
    cross[0] = a[1]*b[2]-a[2]*b[1]
    cross[1] = a[2]*b[0]-a[0]*b[2]
    cross[2] = a[0]*b[1]-a[1]*b[0]
    return Numeric.array(cross)


def ssecoeff(smallmap,model,da,box,counter,name):
	segment=smallmap
	template=model
	maxvalue=-99999999
	bestalt=0
	bestaz=0
	
	altarray=[]
	alt=0
	while alt < 180:
		altarray.append(alt*3.14/180)
		alt=alt+da
	azarray=[]
	az=0
	while az < 360:
		azarray.append(az*3.14/180)
		az=az+da
	for altrot in altarray:
		for azrot in azarray:
			map1=EMAN.EMData()
			map1 = template.copy()
			map1.setRAlign(altrot,azrot,0)
			map1.setTAlign(0,0,0)
			map1.rotateAndTranslate()
			ccf=segment.calcCCF(map1)
			value=ccf.valueAt(box/2,box/2,box/2)
			if value > maxvalue:
				maxvalue=value
				bestalt=altrot*180/3.14
				bestaz=azrot*180/3.14
				#print altrot*180/3.14, azrot*180/3.14, value
				#out="map-%d-%s.mrc"%(counter+1,name)
				#map1.writeImage(out)
	return(bestalt, bestaz, maxvalue)

def chop(bigmap,box,xcoord,ycoord,zcoord,threshold,counter):
	target=bigmap
	chunk=EMAN.EMData()
	chunk.readMRCArea(target,int(xcoord-box/2),int(ycoord-box/2),int(zcoord-box/2),box,box,box)
	chunk.update()
	chunk.realFilter(0,threshold)
	chunk.update()
	#out="chunk-%d.mrc"%(counter+1)
	#chunk.writeImage(out)
	return(chunk)

def createsse(box,apix):
	helix=EMAN.EMData()
	helix.setSize(box,box,box)
	helix.zero()
	sheet=EMAN.EMData()
	sheet.setSize(box,box,box)
	sheet.zero()

	x=0
	while x < box:
		y=0
		while y < box:
			z=0
			while z < box:
				if z <=(box/2+2/apix)  and z >=(box/2-2/apix) :
					if sqrt((x-box/2)**2+(y-box/2)**2) < 8/apix:
						sheet.setValueAt(x,y,z,1)
						sheet.update()
				if z <= (box/2+8/apix)  and z >=(box/2-8/apix):
					if sqrt((x-box/2)**2+(y-box/2)**2) < 3/apix:
						helix.setValueAt(x,y,z,1)
						helix.update()		
				z=z+1
			y=y+1
		x=x+1
	return(helix,sheet)






if (len(argv)<3) :
	print "Usage:\nsshunter.py <input mrc/pdb file> <apix> <res>\n"
	print "########OPTIONS#######\n"
	print "hh=<helixhunter coeff file>    If no interemdiate coeff file is given, a correlation map will be calculated (slow)\n"
	print "thr=<threshold> If input is an mrc, pseudoatoms are generated at a threshold\n"
	print "trans=<translation>  translation\n"

	sys.exit(1)

target=argv[1]
name=target.split('\\')
n=name[-1]
name2=n.split('/')
tname=name2[-1]
t=tname.split('.')
targetName=str(t[0])
targetExt=str(t[-1])
apix=float(argv[2])
res=float(argv[3])

#apix=1
#res=8
thr=0
inthh="none"
finaltrans=0
helixcutoff=2
sheetcutoff=-2
percen=0

for a in argv[4:] :
        s=a.split('=')
	if (s[0]=='trans'):
		finaltrans=float(s[1])
	elif (s[0]=='thr') :
		thr=float(s[1])
	elif (s[0]=='hhthr'):
		hhthr=float(s[1])
	elif (s[0]=='hh'):
		inthh=str(s[1])
	elif (s[0]=='helixcutoff'):
		helixcutoff=float(s[1])
	elif (s[0]=='sheetcutoff'):
		sheetcutoff=float(s[1])	
	elif (s[0]=='percen'):
		percen=1			
        else:
                print("Unknown argument "+a)
                exit(1)

###################################read input data
if (inthh=="none"):
	print inthh
	cmdhh="helixhunter2 %s %s.iv %f da=5 docylccffirst int=int-%s"%(target, targetName, apix, targetName)
	os.system(cmdhh)
	inthh="int-%s-coeff.mrc"%(targetName)

if (targetExt=="mrc"):
	targetMRC=EMAN.EMData()
	targetMRC.readImage(argv[1],-1)
	mapMean=targetMRC.Mean()
	mapSigma=targetMRC.Sigma()
	if thr==0:
		thr=mapMean+4*mapSigma
	print "Calculating threshold"
	print "   mean:      %f"%(mapMean)
	print "   sigma:     %f"%(mapSigma)
	print "   threshold: %f"%(thr)
	target_xsize=targetMRC.xSize()
	target_ysize=targetMRC.ySize()
	target_zsize=targetMRC.zSize()
	pdbOut="seg-%s.pdb"%(targetName)
	cmd0="pseudoatom.py %s %s %f %f %f"%(target, pdbOut, apix, res, thr)
	os.system(cmd0)
	file1=open(pdbOut, "r")
else:
	file1=open(target, "r")
lines1=file1.readlines()
file1.close()
##################################################################



#################################read in coordinates of points
x=[]
y=[]
z=[]
atomNumber=[]
atomCount=0

for line in lines1:
	isatom=str(line[0:6].strip())
	if (isatom=="ATOM"):
		TempX=(float(line[30:38].strip()))/apix
		TempY=(float(line[38:46].strip()))/apix
		TempZ=(float(line[46:54].strip()))/apix
		x.append(TempX)
		y.append(TempY)
		z.append(TempZ)
		atomNumber.append(int(line[6:11].strip()))
		atomCount=atomCount+1
##################################################################



"""
#############################Cross corrletion with prototypical sheet
box=int((15/apix)/2+1)*2
da=5
models=createsse(box,apix)
#helix=models[0]
sheet=models[1]
#besthelix=[]
bestsheet=[]
counter=0

while counter < len(x):	
	smallmap=chop(target,box,x[counter],y[counter],z[counter],thr,counter)
	#hv=ssecoeff(smallmap,helix,da,box,counter,"helix")
	sv=ssecoeff(smallmap,sheet,da,box,counter,"sheet")
	#besthelix.append(hv[2])
	bestsheet.append(sv[2])
	counter=counter+1
	#print "%d of %d sheet: %s"%(counter, len(x), sv[2])
#print MLab.mean(besthelix), MLab.std(besthelix)
sheetmean=MLab.mean(bestsheet)
sheetstd=MLab.std(bestsheet)
"""

########################### Get correlation value at point 
hhMrc=EMAN.EMData()
hhMrc.readImage(inthh,-1)
valueAtArray=[]
avghhvalue=0
hc=0
while hc < atomCount:
	hhvalue=hhMrc.valueAtInterp(x[hc],y[hc],z[hc])
	avghhvalue=hhvalue+avghhvalue
	valueAtArray.append(hhvalue)
	hc=hc+1
avghhvalue=avghhvalue/atomCount
print "Correlation threshold: %f"%(avghhvalue)
##################################################################




########################### all to all distance claclulations
i=0
j=0
distance=[0]*atomCount

while i < atomCount:
	x1=x[i]
	y1=y[i]
	z1=z[i]
	distance[i]=[0]*atomCount
	while (j < atomCount):
		x2=x[j]
		y2=y[j]
		z2=z[j]
		distance[i][j]=(math.sqrt((x1-x2)**2+(y1-y2)**2+(z1-z2)**2))*apix
		j=j+1
	j=0
	i=i+1
##################################################################



########################### set bounding box
k=0
kernelwidth=int(round(7.25/apix))
pixels=[]
while (k < atomCount):
	tempPixel=[]
	pixelCoords=[]
	intX=int(x[k])
	intY=int(y[k])
	intZ=int(z[k])
	for addX in range(-1*kernelwidth, kernelwidth, 1):
		tempX=int((addX+intX))
		for addY in range(-1*kernelwidth, kernelwidth, 1):
			tempY=int((addY+intY))
			for addZ in range(-1*kernelwidth, kernelwidth, 1):
				tempZ=int((addZ+intZ))
				if targetMRC.valueAt(tempX,tempY,tempZ) > thr:
					tempPixel=[tempX,tempY,tempZ]
					if tempPixel not in pixelCoords:
						pixelCoords.append(tempPixel)
	pixels.append(pixelCoords)
	k=k+1
##################################################################





axisMatrix=[]
index1=0
neighborhood=[]
pointcloud=[]
betadistance=8.25
while index1 < atomCount:
########################## Check 2nd moments to see if globular
	NumPoints1=Numeric.array(pixels[index1],'d')
	NumPoints1_mean=Numeric.sum(NumPoints1)/len(NumPoints1)
	NumPoints2=NumPoints1-NumPoints1_mean

	points1=Numeric.sum(map(Numeric.innerproduct,NumPoints2,NumPoints2))
	h = Numeric.sum(map(Numeric.outerproduct,NumPoints2,NumPoints2))
	[u1,x1,v1]=MLab.svd(h)
	if x1==[0,0,0]:
		print index1,
		print "is bad"
		xmod=x1
	else:
		xmod=x1/max(x1)
#	axisMatrix.append(xmod)
	if xmod[2]==0:
		aspectratio=0

	else:
		aspectratio=xmod[1]/xmod[2]
	axisMatrix.append(aspectratio)
#	print "%d: coeff: %f, axis: %s "%(atomNumber[index1],valueAtArray[index1],aspectratio)
##################################################################



##############################checks for nearest atoms and nearest helical atoms
	index2=0
	mindistance1=99998
	mindistance2=99999
	helixneighbor1=0
	helixneighbor2=0
	mindistance3=99998
	mindistance4=99999
	neighbor3=0
	neighbor4=0
	cloud=[]
	while index2 < atomCount:
		if distance[index1][index2]<=betadistance and atomNumber[index1]!=atomNumber[index2]:
			cloud.append(atomNumber[index2])
		if distance[index1][index2] <= mindistance2 and distance[index1][index2]>0.1 and valueAtArray[index2]>=avghhvalue:
			if distance[index1][index2] <= mindistance1:
				mindistance2=mindistance1
				mindistance1=distance[index1][index2]
				helixneighbor2=helixneighbor1
				helixneighbor1=atomNumber[index2]
			else:
				mindistance2=distance[index1][index2]
				helixneighbor2=atomNumber[index2]
				
		if distance[index1][index2] <= mindistance4 and distance[index1][index2]>0.1:
			if distance[index1][index2] <= mindistance3:
				mindistance4=mindistance3
				mindistance3=distance[index1][index2]
				neighbor4=neighbor3
				neighbor3=atomNumber[index2]
#				print "%d is the closest at %f, %f, %f"%(atomNumber[index2],distance[index1][index2],mindistance3,mindistance4)
			else:
				mindistance4=distance[index1][index2]
				neighbor4=atomNumber[index2]
#				print "   %d is the next closest at %f"%(atomNumber[index2],distance[index1][index2])
		index2=index2+1
	neighbors=(helixneighbor1,helixneighbor2,mindistance1,mindistance2,neighbor3, neighbor4)
	pointcloud.append(cloud)
#	print atomNumber[index1], neighbors
	neighborhood.append(neighbors)
	index1=index1+1	
##################################################################




index3=0
helixlist=[]
sheetlist=[]
generallist=[]
while index3 < atomCount:

##############################checks generic angles
	origin=index3
	p1=neighborhood[index3][4]-1
	v1=(x[p1]-x[origin],y[p1]-y[origin],z[p1]-z[origin])
	lengthV1=math.sqrt(v1[0]**2+v1[1]**2+v1[2]**2)
	v1n=(v1[0]/lengthV1,v1[1]/lengthV1,v1[2]/lengthV1)
#	lV1=lengthV1*apix

	p2=neighborhood[index3][5]-1
	v2=(x[p2]-x[origin],y[p2]-y[origin],z[p2]-z[origin])
	lengthV2=math.sqrt(v2[0]**2+v2[1]**2+v2[2]**2)
	v2n=(v2[0]/lengthV2,v2[1]/lengthV2,v2[2]/lengthV2)
#	lV2=lengthV2*apix
	
	dp=Numeric.dot(v1n,v2n)
	if dp > 1:
		dp=1
	if dp<-1:
		dp=-1
	genericAngle=math.acos(dp)*(180/math.pi)
	if genericAngle>90:
		genericAngle=180-genericAngle
##################################################################


##############################checks helix angles
	origin=index3
	p1=neighborhood[index3][0]-1
	v1=(x[p1]-x[origin],y[p1]-y[origin],z[p1]-z[origin])
	lengthV1=math.sqrt(v1[0]**2+v1[1]**2+v1[2]**2)
	v1n=(v1[0]/lengthV1,v1[1]/lengthV1,v1[2]/lengthV1)
#	lV1=lengthV1*apix

	p2=neighborhood[index3][1]-1
	v2=(x[p2]-x[origin],y[p2]-y[origin],z[p2]-z[origin])
	lengthV2=math.sqrt(v2[0]**2+v2[1]**2+v2[2]**2)
	v2n=(v2[0]/lengthV2,v2[1]/lengthV2,v2[2]/lengthV2)
#	lV2=lengthV2*apix
	
	dp=Numeric.dot(v1n,v2n)
	if dp > 1:
		dp=1
	if dp<-1:
		dp=-1
	helixAngle=math.acos(dp)*(180/math.pi)
	if helixAngle>90:
		helixAngle=180-helixAngle
##################################################################



##############################checks sheet angles

	cloud=pointcloud[index3]
	arrayofxp=[]
	for firstpoint in cloud:
		point1=firstpoint-1
		pv1=(x[point1]-x[origin],y[point1]-y[origin],z[point1]-z[origin])
		lengthPV1=math.sqrt(pv1[0]**2+pv1[1]**2+pv1[2]**2)
		#print firstpoint, point1, pv1, lengthPV1
		pv1n=(pv1[0]/lengthPV1,pv1[1]/lengthPV1,pv1[2]/lengthPV1)
		
		for secondpoint in cloud:
			point2=secondpoint-1
			if point2 != point1 and point2 != origin:
				pv2=(x[point2]-x[origin],y[point2]-y[origin],z[point2]-z[origin])
				lengthPV2=math.sqrt(pv2[0]**2+pv2[1]**2+pv2[2]**2)
				pv2n=(pv2[0]/lengthPV2,pv2[1]/lengthPV2,pv2[2]/lengthPV2)
				xp=cross_product(pv1,pv2)
				
				lengthxp=math.sqrt(xp[0]**2+xp[1]**2+xp[2]**2)
				if lengthxp>0:
					xpn=(xp[0]/lengthxp, xp[1]/lengthxp, xp[2]/lengthxp)
				else:
					xpn=(xp[0], xp[1], xp[2])
				arrayofxp.append(xpn)
#				print origin, firstpoint, secondpoint, xpn, lengthxp
	dpxpcounter=0
	dpxpsum=0
#	print "this is array length of %s:  %d "%(atomNumber[index3], len(arrayofxp) )
	if len(arrayofxp) >=2:
		for dpxp1 in arrayofxp:
			for dpxp2 in arrayofxp:
				if dpxp1 != dpxp2:
					dpxp=Numeric.dot(dpxp1,dpxp2)
					if dpxp > 1:
						dpxpAngle=0
					elif dpxp<-1:
						dpxpAngle=180
					else:
						dpxpAngle=math.acos(dpxp)*(180/math.pi)
 					if dpxpAngle>90:
						dpxpAngle=180-dpxpAngle
					
					dpxpcounter=dpxpcounter+1
					dpxpsum=dpxpsum+dpxpAngle
		if dpxpsum==0 and dpxpcounter==0:
			betaAngle=0
		else: 
			betaAngle=dpxpsum/dpxpcounter
	else:
		betaAngle=90	
	generallist.append(genericAngle)
	helixlist.append(helixAngle)
	sheetlist.append(betaAngle)
#	print "%d: coeff: %f, axis: %s, helix angle: %f %s, sheet angle: %f %s "%(atomNumber[index3],valueAtArray[index3],axisMatrix[index3], helixAngle, neighborhood[index3], betaAngle, cloud)
	index3=index3+1
##################################################################



############################Scoring algorithm
scorearray1=[]
scorearray2=[]
helix=[]
sheet=[]
index4=0
scorefile="score-%s.pdb"%(targetName)
outscore=open(scorefile,"w")

scorematrix=Numeric.zeros((atomCount,4), typecode='d')
while index4 < atomCount:
	score=0
	temp_neighbors=neighborhood[index4]
	
	print "Atom Number: %s   "%(atomNumber[index4])
	
	print "      Aspect Ratio:       %s"%(axisMatrix[index4]),
	if axisMatrix[index4] <= 2.5:
		score=score+1
		print " (+1)"
	else:
		score=score-1
		print " (-1)"
#	scorematrix[index4][0]=axisMatrix[index4]
	
	print "      Helix Correlation:  %s"%(valueAtArray[index4]),
	if valueAtArray[index4] >= (0.9*avghhvalue):
		tmpscore=(valueAtArray[index4]/avghhvalue)
		score=score+tmpscore
		print " (+%f)"%(tmpscore)
	else:
		score=score-1
		print " (-1)"	
	
	
	print "      General Angle:      %s"%(generallist[index4]),
	if generallist[index4] <= 45:
		score=score+1
		print " (+1)"
	else:
		score=score-1
		print " (-1)"
		
	print "      Helix Angle:        %s"%(helixlist[index4]), 
	if helixlist[index4] <= 45:
		score=score+1
		print " (+1)"
	else:
		print " (0)"
#		score=score-1

	print "      Sheet Angle:        %s"%(sheetlist[index4]),
	if sheetlist[index4] >= 30 and sheetlist[index4] !=90:
		score=score-1
		print " (-1)"
	elif sheetlist[index4] <= .1:
		score=score+1
		print " (+1)"
	else:
		print " (0)"
#	scorematrix[index4][3]=sheetlist[index4]
			
	print "      Helix Neighbors:    %s"%(str(neighborhood[index4][0:4])),
	if valueAtArray[temp_neighbors[4]-1] >= (0.9*avghhvalue):
		score=score+.5
		print " (+.5)",
	elif valueAtArray[temp_neighbors[4]-1] <= (0.7*avghhvalue):
		score=score-.5	
		print " (-.5)",
	else:
		print " (0)",

	if valueAtArray[temp_neighbors[5]-1] >= (0.9*avghhvalue):
		score=score+.5
		print " (+.5)"
	elif valueAtArray[temp_neighbors[5]-1] <= (0.7*avghhvalue):
		score=score-.5		
		print " (-.5)"
	else:
		print " (0)"

	print "      Closest Neighbors:  %s, %f, %f"%(str(neighborhood[index4][4:]),valueAtArray[temp_neighbors[4]-1],valueAtArray[temp_neighbors[5]-1]),	
	if temp_neighbors[2] <= 9 or temp_neighbors[3] <= 9:
		tmpscore=0.5*(18/(temp_neighbors[2]+temp_neighbors[3]))
		score=score+tmpscore
		print " (+%f)"%(tmpscore)
	elif temp_neighbors[2] >= 9 and temp_neighbors[3] >= 9:
		score=score-1
		print " (-1)"

	print "      Sheet Neighbors:    %s"%(pointcloud[index4]),
	if len(pointcloud[index4]) <= 3:
		score=score+1
		print " (+1)"
	else:
		score=score-1
		print " (-1)"	
		
	print "    Score: %f"%(score) 


#
#	if temp_neighbors[3] <= 9:
#		score=score+1
#	else:
#		score=score-1	
#	print "      Sheet Correlation:  %s"%(bestsheet[index4])	
#	if bestsheet[index4] <= (sheetmean-sheetstd):
#		score=score+1
#	elif bestsheet[index4] >= (sheetmean+sheetstd):
#		score=score-1		

	if percen==0:
		if score >= helixcutoff:
			helix.append(atomNumber[index4])
			print "      In Helix"
		if score <= sheetcutoff:
			sheet.append(atomNumber[index4])
			print "      In Sheet"
			
	outscore.write(lines1[index4][:60]+"%6.2f"%(score)+lines1[index4][66:])
	index4=index4+1
	scorearray1.append(score)
	scorearray2.append(score)

outscore.close()
##################################################################





""" svd for better scoring
u, w, vh = MLab.svd(scorematrix)

wfile = open("sval.dat","w")
for i in range(len(w)):
	wfile.write("%d\t%g\n" % (i, w[i]))
wfile.close()

svalnum = 3
projval = Numeric.zeros((atomCount,svalnum), typecode='d')
for i in range(atomCount):
	atom = scorematrix[i]
	for k in range(svalnum):
		vk = vh[k]
		projval[i][k] = Numeric.dot(atom, vk)

classesnum = 3
clusterid, centroids, error, nfound = Pycluster.kcluster(projval, nclusters=classesnum, transpose=0, npass=1, method='a', dist='e')

print "clusterid =", clusterid
print "centroids=", centroids
outsvd=open("score-svd.pdb","w")
index4=0
while index4 < atomCount:
	outsvd.write(lines1[index4][:60]+"%6.2f"%(clusterid[index4])+lines1[index4][66:])
	index4=index4+1
outsvd.close()
"""




##################################Final automatic assignments
outhelix=open("helix.pdb","w")
outsheet=open("sheet.pdb","w")
spoints=0
hpoints=0

if percen!=0:
	tempHpoints=int((helixcutoff/100)*atomCount)
	print tempHpoints
	while hpoints!= tempHpoints:
		tempmax=max(scorearray1)
		print tempmax
		index=0
		while index < atomCount:
			tempscore=scorearray1[index]
			if tempscore >= tempmax:
				#helix.append(atomNumber[index])
				scorearray1[index]=0
				hpoints=hpoints+1
				outhelix.write(lines1[index])
			index=index+1
	tempSpoints=int((sheetcutoff/100)*atomCount)
	print tempSpoints
	while spoints!= tempSpoints:
		tempmin=min(scorearray2)
		print tempmin
		index=0
		while index < atomCount:
			tempscore=scorearray2[index]
			if tempscore <= tempmin:
				#helix.append(atomNumber[index])
				scorearray2[index]=99999
				spoints=spoints+1
				outsheet.write(lines1[index])
			index=index+1				

else:	
	index5=0
	while index5<atomCount:
		minhelixdistance=99999
		minsheetdistance=99999

		if atomNumber[index5] in helix:
			for point in helix:
				apoint=point-1
				if distance[index5][apoint] < minhelixdistance and apoint!=index5:
					minhelixdistance=distance[index5][apoint]
			if minhelixdistance<8:		
				outhelix.write(lines1[index5])

		elif atomNumber[index5] in sheet:
			for point in sheet:
				bpoint=point-1
				if distance[index5][bpoint] < minsheetdistance and bpoint!=index5:
					minsheetdistance=distance[index5][bpoint]
			if minsheetdistance<9:		
				outsheet.write(lines1[index5])		
		index5=index5+1
outhelix.close()
outsheet.close()
#####################################################################
